package com.informit.mywalks;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.IntentReceiver;
import android.database.Cursor;
import android.location.Location;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

public class MyWalksEdit extends Activity {
    private Uri mURI;

    private Cursor mCursor;

    private LocationManager lm;

    private EditText name;

    private EditText description;

    private static final String LOCATION_CHANGED = "locChanged";

    @Override
    protected void onCreate(Bundle icicle) {
	super.onCreate(icicle);
	setContentView(R.layout.edit);

	final Intent intent = getIntent();
	final String action = intent.getAction();

	if (action.equals(Intent.INSERT_ACTION)) {
	    mURI = getContentResolver().insert(intent.getData(), null);

	    // Will set the result to cancel until we save the values.
	    setResult(RESULT_CANCELED, mURI.toString());
	    if (mURI == null) {
		Log.e("MyWalks", "Failed to insert new geo photo into "
			+ getIntent().getData());
		finish();
		return;
	    }
	}

	mCursor = managedQuery(mURI, null, null, null);

	lm = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);

	name = (EditText) findViewById(R.id.name);
	description = (EditText) findViewById(R.id.description);

	IntentFilter filter = new IntentFilter(LOCATION_CHANGED);
	UpdateWalkReceiver uwr = new UpdateWalkReceiver();
	registerReceiver(uwr, filter);

	final Button track = (Button) findViewById(R.id.tracking);
	track.setOnClickListener(new Button.OnClickListener() {
	    public void onClick(View arg0) {
		Intent intent = new Intent(LOCATION_CHANGED);
		lm.requestUpdates(lm.getProvider("gps"), 5000l, 5f, intent);
	    }
	});

	final Button save = (Button) findViewById(R.id.stoptracking);
	save.setOnClickListener(new Button.OnClickListener() {
	    public void onClick(View arg0) {
		if (mCursor.first()) {
		    mCursor.updateString(1, name.getText().toString());
		    mCursor.updateString(2, description.getText().toString());
		    mCursor.commitUpdates();
		    mCursor.deactivate();
		    mCursor.close();
		}
		setResult(RESULT_OK, mURI.toString());
		MyWalksEdit.this.finish();
	    }
	});

    }

    @Override
    protected void onStop() {
	super.onStop();
	// Unregister intent
    }

    private class UpdateWalkReceiver extends IntentReceiver {

	@Override
	public void onReceiveIntent(Context context, Intent intent) {
	    Location loc = (Location) intent.getParcelableExtra("location");
	    ContentValues values = new ContentValues();
	    values.put("latitude", (int) (loc.getLatitude() * 1E6));
	    values.put("longitude", (int) (loc.getLongitude() * 1E6));

	    getContentResolver().insert(
		    mURI.buildUpon().appendPath("points").build(), values);
	}

    }
}
