package com.informit.mywalks;

import android.app.ListActivity;
import android.content.ContentUris;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

public class MyWalksList extends ListActivity {

    // Menu item Ids
    public static final int MENU_INSERT_ID = Menu.FIRST;

    public static final int MENU_DELETE_ID = Menu.FIRST + 1;

    private Cursor mCursor;

    @Override
    public void onCreate(Bundle icicle) {
	super.onCreate(icicle);
	setContentView(R.layout.list);

	// If no data was given in the intent (because we were started
	// as a MAIN activity), then use our default content provider.
	Intent intent = getIntent();
	if (intent.getData() == null) {
	    intent.setData(Walks.MyWalks.CONTENT_URI);
	}

	mCursor = managedQuery(getIntent().getData(), null, null, null);

	// Now create a new list adapter bound to the cursor.
	// SimpleListAdapter is designed for binding to a Cursor.
	ListAdapter adapter = new SimpleCursorAdapter(this, R.layout.listitem,
		mCursor, new String[] { Walks.MyWalks.NAME,
			Walks.MyWalks.DESCRIPTION }, new int[] { R.id.name,
			R.id.description });

	// Bind to our new adapter.
	setListAdapter(adapter);
	setupFooter();
    }

    private void setupFooter() {
	TextView tv = (TextView) findViewById(R.id.created);
	Uri newUri = Uri
		.parse("content://com.informit.mywalks.Walks/walk?from=1174497980534&to=1174497980536");
	Cursor cursor = getContentResolver().query(newUri,
		new String[] { Walks.MyWalks.NAME }, null, null, null);
	if (cursor.first()) {
	    tv.setText(cursor, 0);
	}
	cursor.close();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
	super.onCreateOptionsMenu(menu);
	menu.add(0, MENU_INSERT_ID, R.string.menu_insert).setShortcut('1', 'n');
	menu.add(0, MENU_DELETE_ID, R.string.menu_delete).setShortcut('2', 'd');
	return true;
    }

    @Override
    public boolean onOptionsItemSelected(Menu.Item item) {
	// Handle all of the possible menu actions.
	switch (item.getId()) {
	case MENU_INSERT_ID:
	    insertItem();
	    break;
	case MENU_DELETE_ID:
	    deleteItem(ContentUris.withAppendedId(getIntent().getData(),
		    getSelectedItemId()));
	    break;
	}
	return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
	// We create a new Uri based on the list index.
	Uri url = ContentUris.withAppendedId(getIntent().getData(), id);
	// We don't need any result back from the new activity
	startActivity(new Intent(Intent.VIEW_ACTION, url));
    }

    private void deleteItem(Uri uri) {
	Cursor del = managedQuery(uri, null, null, null);
	if (del.first()) {
	    del.deleteRow();
	    del.deactivate();
	    del.close();
	}
    }

    private void insertItem() {
	Intent intent = new Intent();
	intent.setData(Walks.MyWalks.CONTENT_URI);
	intent.setAction(Intent.INSERT_ACTION);
	this.startSubActivity(intent, 0);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode,
	    String data, Bundle extras) {
	// The request code is the one with which we started the activity
	// (defined above in insertItem)
	if (requestCode == 0) {
	    // We saved the value in the other activity so we don't need to
	    // do
	    // anything if the activity returned with RESULT_OK
	    if (resultCode == RESULT_OK)
		;
	    // On the other hand, if we did not save any information, lets
	    // delete the newly created field from the db
	    else if (resultCode == RESULT_CANCELED) {
		Log.i("MyLog", data);
		deleteItem(Uri.parse(data));
	    }
	}
	super.onActivityResult(requestCode, resultCode, data, extras);
    }
}