package com.informit.mywalks;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Resources;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

public class WalksProvider extends ContentProvider {

    private SQLiteDatabase mDB;

    private static final String TAG = "MyLog";

    private static final String DATABASE_NAME = "mywalks.db";

    private static final int DATABASE_VERSION = 1;

    private static final int WALK = 1;

    private static final int WALK_ID = 2;

    private static final int WALK_POINTS = 3;

    private static final int WALK_POINTS_ID = 4;

    private static final UriMatcher URI_MATCHER = new UriMatcher(
	    UriMatcher.NO_MATCH);

    private static Map<String, String> WALK_LIST_PROJECTION_MAP = null;

    private static Map<String, String> POINT_LIST_PROJECTION_MAP = null;

    private static class DatabaseHelper extends SQLiteOpenHelper {

	private InputStream in;

	public DatabaseHelper(InputStream in) {
	    this.in = in;
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
	    db
		    .execSQL("CREATE TABLE walks (_id INTEGER PRIMARY KEY AUTOINCREMENT,"
			    + "name TEXT,"
			    + "description TEXT,"
			    + "created INTEGER," + "modified INTEGER" + ");");
	    db
		    .execSQL("CREATE TABLE points (_id INTEGER PRIMARY KEY AUTOINCREMENT,"
			    + "_walk_id INTEGER,"
			    + "latitude INTEGER,"
			    + "longitude INTEGER," + "created INTEGER" + ");");
	    // inserting initial values located under assets/insert.sql
	    insertValues(db);
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
	    Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
		    + newVersion + ", which will destroy all old data");
	    db.execSQL("DROP TABLE IF EXISTS walks");
	    db.execSQL("DROP TABLE IF EXISTS points");
	    onCreate(db);
	}

	public void insertValues(SQLiteDatabase db) {
	    String CurLine = "";
	    InputStreamReader converter = new InputStreamReader(in);
	    BufferedReader in = new BufferedReader(converter);
	    try {
		// read each line of the sql file and insert it into the db
		while ((CurLine = in.readLine()) != null) {
		    db.execSQL(CurLine);
		}
	    } catch (IOException e) {
		Log.e(TAG, "error reading insert values");
	    }
	}
    }

    @Override
    public boolean onCreate() {
	DatabaseHelper dbHelper;
	try {
	    dbHelper = new DatabaseHelper(getContext().getAssets().open(
		    "insert.sql"));
	    mDB = dbHelper.openDatabase(getContext(), DATABASE_NAME, null,
		    DATABASE_VERSION);
	} catch (IOException e) {
	    Log.e(TAG, "error reading insert values");
	}

	return (mDB == null) ? false : true;
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
	int count;
	@SuppressWarnings("unused")
	long rowId = 0;
	int match = URI_MATCHER.match(uri);
	switch (match) {
	case WALK:
	    count = mDB.delete("walks", selection, selectionArgs);
	    count = mDB.delete("points", null, null);
	    break;

	case WALK_POINTS:
	    String segment = uri.getPathSegments().get(1);
	    rowId = Long.parseLong(segment);
	    count = mDB.delete("walks", "_id="
		    + segment
		    + (!TextUtils.isEmpty(selection) ? " AND (" + selection
			    + ')' : ""), selectionArgs);
	    // Also delete all points
	    mDB.delete("points", "_walk_id=" + segment, null);
	    break;

	default:
	    throw new IllegalArgumentException("Unknown URL " + uri);
	}
	getContext().getContentResolver().notifyChange(uri, null);
	return count;
    }

    @Override
    public String getType(Uri uri) {
	int match = URI_MATCHER.match(uri);
	switch (match) {
	case WALK:
	    return "vnd.android.cursor.dir/vnd.informit.walk";
	case WALK_ID:
	    return "vnd.android.cursor.item/vnd.informit.walk";
	case WALK_POINTS:
	    return "vnd.android.cursor.dir/vnd.informit.points";
	case WALK_POINTS_ID:
	    return "vnd.android.cursor.item/vnd.informit.points";
	default:
	    throw new IllegalArgumentException("Unknown URL " + uri);
	}
    }

    @Override
    public Uri insert(Uri uri, ContentValues values) {
	int match = URI_MATCHER.match(uri);
	switch (match) {
	case WALK:
	    return insertWalk(values);
	case WALK_POINTS:
	    values.put(Walks.MyPoints._WALK_ID, uri.getPathSegments().get(1));
	    return insertPoint(values);
	default:
	    throw new IllegalArgumentException("Unknown URL " + uri);
	}
    }

    /*
         * Insert a new walk
         */
    public Uri insertWalk(ContentValues initialValues) {
	long rowID;
	ContentValues values;
	if (initialValues != null) {
	    values = new ContentValues(initialValues);
	} else {
	    values = new ContentValues();
	}
	Long now = Long.valueOf(System.currentTimeMillis());
	Resources r = Resources.getSystem();

	if (values.containsKey(Walks.MyWalks.NAME) == false) {
	    values.put(Walks.MyWalks.NAME, r
		    .getString(android.R.string.untitled));
	}

	if (values.containsKey(Walks.MyWalks.DESCRIPTION) == false) {
	    values.put(Walks.MyWalks.NAME, "");
	}

	if (values.containsKey(Walks.MyWalks.CREATED_DATE) == false) {
	    values.put(Walks.MyWalks.CREATED_DATE, now);
	}

	if (values.containsKey(Walks.MyWalks.MODIFIED_DATE) == false) {
	    values.put(Walks.MyWalks.MODIFIED_DATE, now);
	}
	rowID = mDB.insert("walks", "walk", values);
	if (rowID > 0) {
	    mDB.update("walks", values, "_id=" + rowID, null);
	    Uri uri = ContentUris.appendId(
		    Walks.MyWalks.CONTENT_URI.buildUpon(), rowID).build();
	    getContext().getContentResolver().notifyChange(uri, null);
	    return uri;
	}
	throw new SQLException("Failed to insert row");
    }

    public Uri insertPoint(ContentValues values) {
	long rowID;
	Long now = Long.valueOf(System.currentTimeMillis());

	// We need to ensure we have the latitude and longitude set
	if (values.containsKey(Walks.MyPoints.LATITUDE) == false
		|| values.containsKey(Walks.MyPoints.LONGITUDE) == false) {
	    Log
		    .e(TAG,
			    "the latitude and longitude need to be set in order to insert a point.");
	}

	if (values.containsKey(Walks.MyPoints.CREATED_DATE) == false) {
	    values.put(Walks.MyPoints.CREATED_DATE, now);
	}
	rowID = mDB.insert("points", "point", values);
	if (rowID > 0) {
	    mDB.update("points", values, "_id=" + rowID, null);
	    Uri uri = ContentUris.appendId(
		    Walks.MyWalks.CONTENT_URI.buildUpon().appendEncodedPath(
			    values.getAsString(Walks.MyPoints._WALK_ID))
			    .appendEncodedPath("points"), rowID).build();

	    getContext().getContentResolver().notifyChange(uri, null);
	    return uri;
	}
	throw new SQLException("Failed to insert row");
    }

    @Override
    public Cursor query(Uri uri, String[] projection, String selection,
	    String[] selectionArgs, String sortOrder) {
	SQLiteQueryBuilder qb = new SQLiteQueryBuilder();

	switch (URI_MATCHER.match(uri)) {
	case WALK:
	    qb.setTables("walks");
	    qb.setProjectionMap(WALK_LIST_PROJECTION_MAP);
	    break;
	case WALK_ID:
	    qb.setTables("walks");
	    qb.appendWhere("_id=" + uri.getPathSegments().get(1));
	    break;
	case WALK_POINTS:
	    qb.setTables("points");
	    qb.appendWhere("_walk_id=" + uri.getPathSegments().get(1));
	    qb.setProjectionMap(POINT_LIST_PROJECTION_MAP);
	    break;
	case WALK_POINTS_ID:
	    qb.setTables("points");
	    qb.appendWhere("_walk_id=" + uri.getPathSegments().get(1));
	    qb.appendWhere("_id=" + uri.getPathSegments().get(3));
	    break;
	default:
	    throw new IllegalArgumentException("Unknown URL " + uri);
	}

	// Specific Uris
	String from = uri.getQueryParameter("from");
	String to = uri.getQueryParameter("to");
	if (from != null) {
	    qb.appendWhere("created>" + from);
	}
	if (to != null) {
	    // There seems to be a bug here. We need to manually ensure the
	    // AND is part of the where clause
	    qb.appendWhere(" AND created<" + to);
	}

	Cursor c = qb.query(mDB, projection, selection, selectionArgs, null,
		null, null);
	c.setNotificationUri(getContext().getContentResolver(), uri);
	return c;
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection,
	    String[] selectionArgs) {
	// never used
	return 0;
    }

    static {
	URI_MATCHER.addURI("com.informit.mywalks.Walks", "walk", WALK);
	URI_MATCHER.addURI("com.informit.mywalks.Walks", "walk/#", WALK_ID);
	URI_MATCHER.addURI("com.informit.mywalks.Walks", "walk/#/points",
		WALK_POINTS);
	URI_MATCHER.addURI("com.informit.mywalks.Walks", "walk/#/points/#",
		WALK_POINTS_ID);

	WALK_LIST_PROJECTION_MAP = new HashMap<String, String>();
	WALK_LIST_PROJECTION_MAP.put(Walks.MyWalks._ID, "_id");
	WALK_LIST_PROJECTION_MAP.put(Walks.MyWalks.NAME, "name");
	WALK_LIST_PROJECTION_MAP.put(Walks.MyWalks.DESCRIPTION, "description");
	WALK_LIST_PROJECTION_MAP.put(Walks.MyWalks.CREATED_DATE, "created");
	WALK_LIST_PROJECTION_MAP.put(Walks.MyWalks.MODIFIED_DATE, "modified");

	POINT_LIST_PROJECTION_MAP = new HashMap<String, String>();
	POINT_LIST_PROJECTION_MAP.put(Walks.MyPoints._ID, "_id");
	POINT_LIST_PROJECTION_MAP.put(Walks.MyPoints._WALK_ID, "_walk_id");
	POINT_LIST_PROJECTION_MAP.put(Walks.MyPoints.LATITUDE, "latitude");
	POINT_LIST_PROJECTION_MAP.put(Walks.MyPoints.LONGITUDE, "longitude");
	POINT_LIST_PROJECTION_MAP.put(Walks.MyPoints.CREATED_DATE, "created");

    }
}
